#include <stdbool.h>
#include <assert.h>

#include "db_vdp.h"
#include "db_math.h"
#include "db_io.h"

#define QOI_IMPLEMENTATION
#include "qoi.h"

uint32_t brickTextureHandle;

void tick() {
    vdp_Vertex triangle[3] = {
        {
            0.0f, 0.5f, 0.0f, 1.0f,
            1.0f, 0.0f, 0.0f, 1.0f,
            0.0f, 0.0f, 0.0f, 0.0f,
            0.5f, 0.0f, 0.0f, 0.0f,
        },
        {
            -0.5f, -0.5f, 0.0f, 1.0f,
            0.0f, 1.0f, 0.0f, 1.0f,
            0.0f, 0.0f, 0.0f, 0.0f,
            0.0f, 1.0f, 0.0f, 0.0f,
        },
        {
            0.5f, -0.5f, 0.0f, 1.0f,
            0.0f, 0.0f, 1.0f, 1.0f,
            0.0f, 0.0f, 0.0f, 0.0f,
            1.0f, 1.0f, 0.0f, 0.0f,
        },
    };

    vdp_Color32 clearColor = (vdp_Color32){128, 128, 255, 255};
    vdp_clearColor(&clearColor);
    vdp_clearDepth(1.0f);

    vdp_blendEquation(VDP_FUNC_ADD);
    vdp_blendFunc(VDP_BLEND_ONE, VDP_BLEND_ZERO);

    vdp_depthWrite(false);
    vdp_depthFunc(VDP_COMPARE_ALWAYS);

    Mat4 projection = mat4_projectionOrthoAspect(640.0f / 480.0f, 1.0f, 0.0f, 1.0f);
    mat4_loadSIMD(&projection);
    mat4_transformSIMD(&triangle[0].position, &triangle[0].position, 3, sizeof(vdp_Vertex));

    vdp_bindTexture(brickTextureHandle);
    vdp_drawGeometry(VDP_TOPOLOGY_TRIANGLES, 0, 3, triangle);
}

int main() {
    // load texture into memory
    IOFILE *brickTexFile = fs_open("/cd/content/brickTex.qoi", IO_FILEMODE_READ);
    assert(brickTexFile != NULL && "Failed to open brickTex.qoi");

    fs_seek(brickTexFile, 0, IO_WHENCE_END);
    uint32_t brickTexFileLen = fs_tell(brickTexFile);
    fs_seek(brickTexFile, 0, IO_WHENCE_BEGIN);

    uint8_t *brickTexFileBuf = malloc(brickTexFileLen);
    fs_read(brickTexFile, brickTexFileBuf, brickTexFileLen);
    fs_close(brickTexFile);

    // decode QOI image
    qoi_desc desc;
    void *pixelData = qoi_decode(brickTexFileBuf, brickTexFileLen, &desc, 4);
    free(brickTexFileBuf);
    assert(pixelData != NULL && "Failed decoding brickTex.qoi");

    // allocate GPU texture
    brickTextureHandle = vdp_allocTexture(false, VDP_TEXFMT_RGBA8888, desc.width, desc.height);
    assert(brickTextureHandle != -1 && "Failed allocating GPU texture");

    vdp_setTextureData(brickTextureHandle, 0, pixelData, desc.width * desc.height * 4);
    QOI_FREE(pixelData);

    vdp_setVsyncHandler(&tick);
    return 0;
}